@@ -21,7 +21,10 @@
 
 #ifdef HAVE_ALSA
 # include "audio_alsa.c"
-#else /* !HAVE_ALSA */
-# include "audio_oss.c"
-#endif /* !HAVE_ALSA */
-
+#else
+# ifdef MAC_OS_X /* MacOSX */
+#  include "audio_osx.c"
+# else
+#  include "audio_oss.c"
+# endif 
+#endif
