@@ -775,7 +775,7 @@ void flush_wavefile_data(void)
 int process_audio(gfloat *pL, gfloat *pR)
 {
     int len = 0 ;
-    int i ;
+    int i, frame ;
     short *p_short ;
     int *p_int ;
     unsigned char  *p_char ;
@@ -783,6 +783,7 @@ int process_audio(gfloat *pL, gfloat *pR)
     extern int audio_playback ;
     long n_samples_to_read, n_read ;
     double maxpossible ;
+    double feather_out_N ;
     int feather_out = 0 ;
 
     *pL = 0.0 ;
@@ -827,27 +828,39 @@ int process_audio(gfloat *pL, gfloat *pR)
 	n_read = sf_readf_int(sndfile, p_int, n_samples_to_read) ;
     }
 
-    if(playback_samples_remaining - n_read < 1) feather_out = 1 ;
+#define FEATHER_WIDTH 30000
+    if(playback_samples_remaining - n_read < 1) {
+	feather_out = 1 ;
+	feather_out_N = MIN(n_read, FEATHER_WIDTH) ;
+	printf("Feather out n_read=%d, playback_samples_remaining=%d, N=%lf\n", n_read, playback_samples_remaining, feather_out_N) ;
+    }
 
-    for(i = 0  ; i < n_read ; i++) {
+    for(frame = 0  ; frame < n_read ; frame++) {
 	int vl, vr ;
+	i = frame*2 ;
 
 	if(BYTESPERSAMPLE < 3) {
-	    if(feather_out == 1 && n_read-(i+1) < 100) {
-		int j = (n_read-(i))-1 ;
-		double p = (double)(j)/99.0 ;
+	    if(feather_out == 1 && n_read-(frame+1) < FEATHER_WIDTH) {
+		int j = ((n_read-(frame))-1) ;
+		double p = (double)(j)/feather_out_N ;
+
 		if(i > n_read - 100) {
 		    //printf("j:%d %lf %hd %hd ", j, p, p_short[i], p_short[i+1]) ;
 		}
+
 		p_short[i] *= p ;
 		p_short[i+1] *= p ;
-		if(i > n_read - 100) {
-		    //printf("%hd %hd\n", p_short[i], p_short[i+1]) ;
-		}
-		//if(i == n_read-1) printf(" %lf", p) ;
+
+		//if(i > n_read - 100) {
+		//    printf("%hd %hd\n", p_short[i], p_short[i+1]) ;
+		//}
+
+		if(frame == n_read-1) printf("Feather out final %lf, n_read=%d", p, n_read) ;
 	    }
+
 	    vl = p_short[i] ;
 	    vr = p_short[i+1] ;
+
 	} else {
 	    if(feather_out == 1 && n_read-(i+1) < 10000) {
 		double p = 1.0 - (double)(n_read-(i+1))/9999.0 ;
@@ -855,6 +868,7 @@ int process_audio(gfloat *pL, gfloat *pR)
 		p_int[i] *= p ;
 		p_int[i+1] *= p ;
 	    }
+
 	    vl = p_int[i] ;
 	    vr = p_int[i+1] ;
 	}
@@ -863,7 +877,6 @@ int process_audio(gfloat *pL, gfloat *pR)
 	if(-vl > maxl) maxl = -vl ;
 
 	if(stereo) {
-	    i++ ;
 	    if(vr > maxr) maxr = vr ;
 	    if(-vr > maxr) maxr = -vr ;
 	} else {
@@ -883,11 +896,23 @@ int process_audio(gfloat *pL, gfloat *pR)
 	len = audio_device_write(p_char, len) ;
 	playback_position += n_read ;
 	playback_samples_remaining -= n_read ;
+
 	if(playback_samples_remaining < 1) {
 	    extern int audio_is_looping ;
+
 	    if(audio_is_looping == FALSE) {
+		char zeros[1024] ;
+		long zeros_needed ;
+		memset(zeros,0,sizeof(zeros)) ;
 		audio_state = AUDIO_IS_PLAYBACK ;
 		audio_playback = FALSE ;
+
+		zeros_needed = playback_bytes_per_block - (playback_total_bytes % playback_bytes_per_block) ;
+		do {
+		    len = audio_device_write(zeros, MIN(zeros_needed, sizeof(zeros))) ;
+		    zeros_needed -= len ;
+		} while (len >= 0 && zeros_needed > 0) ;
+
 		g_print("Stop playback with playback_samples_remaining:%ld\n", playback_samples_remaining) ;
 		return 1 ;
 	    } else {
@@ -926,6 +951,10 @@ void stop_playback(int force)
 	usleep(100) ;
     }
 
+/*      fprintf(stderr, "Usleeping 300000\n") ;  */
+/*      usleep(300000) ;  */
+/*      fprintf(stderr, "Done usleeping 300000\n") ;  */
+
     audio_state = AUDIO_IS_IDLE ;
     audio_device_close() ;
 }
