@@ -70,7 +70,86 @@ void stop_timer(char *message)
 
 
 
+double bark_z[DENOISE_MAX_FFT] ;
 double window_coef[DENOISE_MAX_FFT] ;
+double sum_window_wgts[DENOISE_MAX_FFT] ;
+double jg_upper[DENOISE_MAX_FFT][11] ;
+double jg_lower[DENOISE_MAX_FFT][11] ;
+
+void compute_bark_z(int FFT_SIZE, int rate)
+{
+    int k ;
+
+    /* compute the bark z value for this frequency bin */
+    for(k = 1 ; k <= FFT_SIZE/2 ; k++) {
+	double freq = (double)rate / 2.0 /(double)(FFT_SIZE/2)*(double)k ;
+	bark_z[k] = 7.0*log(freq/650.0 + sqrt(1 + (freq/650.0)*(freq/650.0))) ;
+    }
+}
+
+void compute_johnston_gain(int FFT_SIZE, double tonality_factor)
+{
+    int k ;
+
+    for (k = 1; k <= FFT_SIZE/2 ; ++k) {
+	int j ;
+
+	for(j = k-1 ; j > 0 ; j--) {
+	    double bark_diff = bark_z[k] - bark_z[j] ;
+
+	    double johnston = 15.81 + 7.5*(bark_diff+.474) - 17.5*sqrt(1.0+(bark_diff+0.474)*(bark_diff+0.474)) ;
+	    double johnston_masked = johnston - (tonality_factor*(14.5+bark_z[j])+5.5*(1.0 - tonality_factor)) ;
+	    double gain = pow(10.0, johnston_masked/10.0) ;
+
+	    jg_lower[k][k-j] = gain ;
+
+	    if(k - j > 10) break ;
+	}
+
+	for(j = k ; j <= FFT_SIZE/2 ; j++) {
+	    double bark_diff = bark_z[j] - bark_z[k] ;
+
+	    double johnston = 15.81 + 7.5*(bark_diff+.474) - 17.5*sqrt(1.0+(bark_diff+0.474)*(bark_diff+0.474)) ;
+	    double johnston_masked = johnston - (tonality_factor*(14.5+bark_z[j])+5.5*(1.0 - tonality_factor)) ;
+	    double gain = pow(10.0, johnston_masked/10.0) ;
+
+	    jg_upper[k][j-k] = gain ;
+
+	    if(j - k > 10) break ;
+
+	}
+    }
+}
+
+int get_window_delta(struct denoise_prefs *pDnprefs)
+{
+
+    if(pDnprefs->window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
+	return pDnprefs->FFT_SIZE/2 ;
+    } else {
+	if(pDnprefs->window_type == DENOISE_WINDOW_BLACKMAN)
+	    return pDnprefs->FFT_SIZE/pDnprefs->smoothness ;
+	else
+	    return 3*pDnprefs->FFT_SIZE/4 ;
+    }
+}
+
+void compute_sum_window_wgts(struct denoise_prefs *pDnprefs)
+{
+    int delta = get_window_delta(pDnprefs) ;
+    int i, k ;
+
+    for(i = 0 ; i < pDnprefs->FFT_SIZE ; i++) {
+	sum_window_wgts[i] = 0.0 ;
+	for(k = i ; k < pDnprefs->FFT_SIZE+i ; k += delta) {
+	    sum_window_wgts[i] += window_coef[k%pDnprefs->FFT_SIZE] ;
+	}
+    }
+
+/*      for(i = 0 ; i < pDnprefs->FFT_SIZE ; i++) {  */
+/*  	g_print("i:%d sww:%lf\n", i, sum_window_wgts[i]) ;  */
+/*      }  */
+}
 
 double gain_weiner(double Yk2, double Dk2)
 {
@@ -95,18 +174,16 @@ double gain_power_subtraction(double Yk2, double Dk2)
 	return 0.0 ;
 }
 
-double gain_lorber(double snr)
+double alpha_lorber(double snr)
 {
-    double snr_db = log10(snr) ;
+    double snr_db = 10.*log10(snr) ;
+    double alpha ;
 
-    double alpha = MAX((3.0 - 0.10*snr_db), 1.0) ;
+    if(snr_db > 20) return 1.0 ;
 
-    double gain = sqrt(1.0 - alpha/(snr+1)) ;
+    alpha = MIN((3.0 - 0.10*snr_db), 3.5) ;
 
-    if(gain > DBL_MIN)
-	return gain ;
-    else
-	return 0.0 ;
+    return alpha ;
 }
 
 #define SLOW_EM
@@ -136,9 +213,9 @@ double gain_em(double Rprio, double Rpost, double alpha)
 {
     /* Ephraim-Malah noise suppression, from Godsill and Wolfe 2001 paper */
     double r = MAX(Rprio/(1.0+Rprio),DBL_MIN) ;
-    double V = (1.0+Rpost)/r  ;
+    double V = Rprio/(1.+Rprio)*Rpost ;
 
-    return sqrt( (1.0+V)/Rpost * r  ) ;
+    return sqrt( r * (1.0+V)/Rpost ) ;
 }
 #endif
 
@@ -210,23 +287,36 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
 		      struct denoise_prefs *pPrefs, int ch)
 {
     int k ;
-    fftw_real noise2[DENOISE_MAX_FFT] ;
+    fftw_real noise2[DENOISE_MAX_FFT/2+1] ;
+    fftw_real noise[DENOISE_MAX_FFT/2+1] ;
     fftw_real Y2[DENOISE_MAX_FFT/2+1] ;
-    fftw_real maskedY2[DENOISE_MAX_FFT/2+1] ;
+    fftw_real Y[DENOISE_MAX_FFT/2+1] ;
+    fftw_real masked[DENOISE_MAX_FFT/2+1] ;
     fftw_real gain_k[DENOISE_MAX_FFT] ;
     static fftw_real bsig_prev[2][DENOISE_MAX_FFT],bY2_prev[2][DENOISE_MAX_FFT/2+1],bgain_prev[2][DENOISE_MAX_FFT/2+1] ;
     fftw_real *sig_prev,*Y2_prev,*gain_prev ;
+    static int debug_frame = 1 ;
+    double SFM, tonality_factor ;
 
     sig_prev = bsig_prev[ch] ;
     Y2_prev = bY2_prev[ch] ;
     gain_prev = bgain_prev[ch] ;
 
+    if(0 || pPrefs->window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
+	/* with overlap-add pre-window the sample */
+	for(k = 0 ; k < pPrefs->FFT_SIZE ; k++) {
+	    windowed[k] = sample[k]*window_coef[k] ;
+	    //if(debug_frame == 3) g_print("k:%d window_coef:%lf windowed:%lf\n", k, window_coef[k], windowed[k]) ;
+	}
 
-    for(k = 0 ; k < pPrefs->FFT_SIZE ; k++) {
-	windowed[k] = sample[k]*window_coef[k] ;
-	noise2[k] = noise_max2[k] ;
-	noise2[k] = noise_min2[k] + 0.5*(noise_max2[k] - noise_min2[k]) ;
-	noise2[k] = noise_avg2[k] ;
+    } else {
+	/* with other methods don't window the sample for FFT,
+ 	   but window the result back into the original
+	   sample the FFT noise-removal process */
+
+	for(k = 0 ; k < pPrefs->FFT_SIZE ; k++) {
+	    windowed[k] = sample[k] ;
+	}
     }
 
 #ifdef HAVE_FFTW3
@@ -235,10 +325,37 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
     rfftw_one(*pFor, windowed, out);
 #endif /* HAVE_FFTW3 */
 
-    for (k = 1; k <= pPrefs->FFT_SIZE/2 ; ++k) {
-	Y2[k] = k < pPrefs->FFT_SIZE/2 ?  out[k]*out[k] + out[pPrefs->FFT_SIZE-k]*out[pPrefs->FFT_SIZE-k] : out[k]*out[k] ;
+    {
+	double sum_log_p = 0.0 ;
+	double sum_p = 0.0 ;
+	double kinv = 1./(double)(pPrefs->FFT_SIZE/2.0) ;
+
+	for (k = 1; k <= pPrefs->FFT_SIZE/2 ; ++k) {
+	    noise2[k] = noise_max2[k] ;
+	    noise2[k] = noise_min2[k] + 0.5*(noise_max2[k] - noise_min2[k]) ;
+	    noise2[k] = noise_avg2[k] ;
+	    noise[k] = sqrt(noise2[k]) ;
+	    if(k < pPrefs->FFT_SIZE/2) {
+		Y2[k] = out[k]*out[k] + out[pPrefs->FFT_SIZE-k]*out[pPrefs->FFT_SIZE-k] ;
+		Y[k] = sqrt(Y2[k]) ;
+	    } else {
+		Y2[k] = out[k]*out[k] ;
+		Y[k] = out[k] ;
+	    }
+	    sum_log_p += log10(Y2[k]) ;
+	    sum_p += Y2[k] ;
+	}
+
+
+	SFM = 10.0*( kinv*sum_log_p - log10(sum_p*kinv) ) ;
+	tonality_factor = MIN(SFM/-60.0, 1) ;
     }
 
+
+    if(pPrefs->noise_suppression_method == DENOISE_LORBER) tonality_factor = 0.0 ;
+
+/*      g_print("SFM:%f tonality:%lf\n", SFM, tonality_factor) ;  */
+
     if(pPrefs->noise_suppression_method == DENOISE_LORBER) {
 	for (k = 1; k <= pPrefs->FFT_SIZE/2 ; ++k) {
 	    double sum = 0 ;
@@ -250,15 +367,59 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
 
 	    for(j = j1 ; j <= j2 ; j++) {
 		double d = ABS(j-k)+1.0 ;
-		double wgt = 1./d ;
-		sum += Y2[k]*wgt ;
+		double wgt = 1./sqrt(d) ;
+		sum += Y2[j]*wgt ;
 		sum_wgts += wgt ;
 	    }
 
-	    maskedY2[k] = sum / (sum_wgts+1.e-300) ;
+	    masked[k] = sum / (sum_wgts+1.e-300) ;
 	}
     }
 
+/*      if(pPrefs->noise_suppression_method == DENOISE_LORBER || pPrefs->noise_suppression_method == DENOISE_WOLFE_GODSILL) {  */
+    if(pPrefs->noise_suppression_method == DENOISE_WOLFE_GODSILL) {
+	for (k = 1; k <= pPrefs->FFT_SIZE/2 ; ++k) {
+	    int j ;
+
+	    masked[k] = 0.0 ;
+
+	    for(j = k-1 ; j > 0 ; j--) {
+#ifdef OLD_N_SLOW
+		double bark_diff = bark_z[k] - bark_z[j] ;
+
+		double johnston = 15.81 + 7.5*(bark_diff+.474) - 17.5*sqrt(1.0+(bark_diff+0.474)*(bark_diff+0.474)) ;
+		double johnston_masked = johnston - (tonality_factor*(14.5+bark_z[j])+5.5*(1.0 - tonality_factor)) ;
+		double gain = pow(10.0, johnston_masked/10.0) ;
+
+		if(gain < 1.e-2) break ;
+#else
+		double gain = jg_lower[k][k-j] ;
+#endif
+		if(k - j > 10) break ;
+
+		masked[k] += MAX((Y2[j]-noise2[j]),0.0)*gain ;
+	    }
+
+	    for(j = k ; j <= pPrefs->FFT_SIZE/2 ; j++) {
+#ifdef OLD_N_SLOW
+		double bark_diff = bark_z[j] - bark_z[k] ;
+
+		double johnston = 15.81 + 7.5*(bark_diff+.474) - 17.5*sqrt(1.0+(bark_diff+0.474)*(bark_diff+0.474)) ;
+		double johnston_masked = johnston - (tonality_factor*(14.5+bark_z[j])+5.5*(1.0 - tonality_factor)) ;
+		double gain = pow(10.0, johnston_masked/10.0) ;
+#else
+		double gain = jg_upper[k][j-k] ;
+#endif
+
+		if(gain < 1.e-2) break ;
+		if(j - k > 10) break ;
+
+		masked[k] += MAX((Y2[j]-noise2[j]),0.0)*gain ;
+	    }
+
+/*  	    if(debug_frame == 3) g_print("k:%d Y:%lf masked:%lf bark:%lf\n", k, Y[k], masked[k], bark_z[k]) ;  */
+	}
+    }
 
 	
 
@@ -318,27 +479,51 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
 		    double Rprio ;
 
 		    if(prev_sample[ch] == 1)
-			Rprio = (1.0-alpha)*MAX(Rpost, 0.0)+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
+			Rprio = (1.0-alpha)*Rpost+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
 		    else
-			Rprio = MAX(Rpost, 0.0) ;
+			Rprio = Rpost ;
 
 		    gain = gain_em(Rprio, Rpost, alpha) ;
     /*  		g_print("Rpost:%lg Rprio:%lg gain:%lg gain_prev:%lg y2_prev:%lg\n", Rpost, Rprio, gain, gain_prev[k], Y2_prev[k]) ;  */
 		    gain_prev[k] = gain ;
 		    Y2_prev[k] = Y2[k] ;
 		} else if(pPrefs->noise_suppression_method == DENOISE_LORBER) {
-		    double SNRlocal = maskedY2[k]/noise2[k]-1.0 ;
+		    double SNRlocal = MAX(masked[k]/noise2[k]-1.0, 0.0) ;
 		    double SNRfilt, SNRprio ;
+		    double alpha ;
 
 		    if(prev_sample[ch] == 1) {
+			double eta = (1.0 + pPrefs->dn_gamma) / 2.0 ;  /* eta seems to be better closer to 1.0 than gamma */
 			SNRfilt = (1.-pPrefs->dn_gamma)*SNRlocal + pPrefs->dn_gamma*(sig_prev[k]/noise2[k]) ;
 			SNRprio = SNRfilt ; /* note, could use another parameter, like pPrefs->dn_gamma, here to compute SNRprio */
+			SNRprio = (1.-eta)*SNRlocal + eta*(sig_prev[k]/noise2[k]) ;
 		    }else {
 			SNRfilt = SNRlocal ;
 			SNRprio = SNRfilt ;
 		    }
-		    gain = gain_lorber(SNRprio) ;
+		    alpha = alpha_lorber(SNRprio) ;
+		    gain = 1.0 - alpha/(SNRfilt+1) ;
 		    sig_prev[k] = MAX(Y2[k]*gain,0.0) ;
+		} else if(pPrefs->noise_suppression_method == DENOISE_WOLFE_GODSILL) {
+		    double Rpost = MAX(Y2[k]/noise2[k]-1.0, 0.0) ;
+		    double alpha = pPrefs->dn_gamma ;
+		    double Rprio ;
+
+		    if(prev_sample[ch] == 1)
+			Rprio = (1.0-alpha)*Rpost+alpha*gain_prev[k]*gain_prev[k]*Y2_prev[k]/noise2[k] ;
+		    else
+			Rprio = Rpost ;
+
+		    Y2_prev[k] = Y2[k] ;
+
+		    if(Y2[k] > masked[k]) {
+			gain = MAX(masked[k]/Y2[k], Rprio/(Rprio+1.0)) ;
+		    } else {
+			gain = 1.0 ;
+		    }
+
+		    gain_prev[k] = gain ;
+
 		} else if(pPrefs->noise_suppression_method == DENOISE_WEINER)
 		    gain = gain_weiner(Y2[k], noise2[k]) ;
 		else
@@ -369,13 +554,8 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
     for(k = 0 ; k < pPrefs->FFT_SIZE ; k++)
 	windowed[k] /= (double)(pPrefs->FFT_SIZE) ;
 
-    if(pPrefs->window_type != DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
-	/* merge results back into sample data based on window function */
-	for(k = 0 ; k < pPrefs->FFT_SIZE ; k++) {
-	    double w = window_coef[k] ;
-	    sample[k] = (1.0-w) * sample[k] + windowed[k] ;
-	}
-    } else {
+    if(0|| pPrefs->window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
+	/** HANNING - OVERLAP - ADD */
 	/* make sure the tails of the sample approach zero magnitude */
 	double offset = windowed[0] ;
 	double hs1 = pPrefs->FFT_SIZE/2 - 1 ;
@@ -390,16 +570,26 @@ void fft_remove_noise(fftw_real sample[], fftw_real noise_min2[], fftw_real nois
 	    double p = ((double)k-hs1)/hs1 ;
 	    sample[k] = windowed[k] - offset*p ;
 	}
+    } else {
+	/* merge results back into sample data based on window function */
+	for(k = 0 ; k < pPrefs->FFT_SIZE ; k++) {
+	    double w = window_coef[k] ;
+	    sample[k] = (1.0-w)*sample[k] + w*windowed[k] ;
+	}
     }
 
     prev_sample[ch] = 1 ;
+    debug_frame++ ;
 }
 
 int denoise_normalize = 0 ;
 
+#define NO_DEBUG
 #ifdef DEBUG
 void print_denoise(char *header, struct denoise_prefs *pDnprefs)
 {
+    int k ;
+
     g_print("******** %s ************\n", header) ;
     g_print("FFT_SIZE:%d\n", pDnprefs->FFT_SIZE) ;
     g_print("n_noise_samples:%d\n", pDnprefs->n_noise_samples) ;
@@ -417,8 +607,12 @@ void print_denoise(char *header, struct denoise_prefs *pDnprefs)
 	case DENOISE_LORBER : g_print("Lorber-Hoeldrich\n") ; break ;
 	case DENOISE_WEINER : g_print("Weiner\n") ; break ;
 	case DENOISE_EM : g_print("Ephram\n") ; break ;
+	case DENOISE_WOLFE_GODSILL : g_print("Wolfe-Godsill\n") ; break ;
 	default : g_print("Spectral Subtraction\n") ; break ;
     }
+    for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
+	g_print("k:%d wc:%lg\n", k, window_coef[k]) ;
+    }
 }
 #else
 void print_denoise(char *header, struct denoise_prefs *pDnprefs) {}
@@ -530,11 +724,13 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
 			window_coef[k] = 1.0 ;
 		    }
 		}
+		compute_sum_window_wgts(pDnprefs) ;
 		framenum = 1 ;
 	    } else if(framenum == 1) {
 		for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
 		    window_coef[k] = fft_window(k,pDnprefs->FFT_SIZE, pDnprefs->window_type) ;
 		}
+		compute_sum_window_wgts(pDnprefs) ;
 		framenum = 2 ;
 	    }
 
@@ -563,19 +759,14 @@ int denoise(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs, long noi
 			right_prev_frame[k+pDnprefs->FFT_SIZE/2] = right[k+pDnprefs->FFT_SIZE/2] ;
 		    }
 		}
-/*  		write_fft_real_wavefile_data(left, right, current, MIN(tmplast, last_sample)) ;  */
+
 		write_fft_real_wavefile_data(left, right, current, current+pDnprefs->FFT_SIZE/2-1) ;
 
-		current += pDnprefs->FFT_SIZE/2 ;
 	    } else {
 		write_fft_real_wavefile_data(left, right, current, MIN(tmplast, last_sample)) ;
-
-		if(pDnprefs->window_type == DENOISE_WINDOW_BLACKMAN)
-		    current += pDnprefs->FFT_SIZE/pDnprefs->smoothness ;
-		else
-		    current += 3*pDnprefs->FFT_SIZE/4 ;
 	    }
 
+	    current += get_window_delta(pDnprefs) ;
 
 	}
     }
@@ -621,7 +812,7 @@ int print_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnpref
 
     for(k = 1 ; k <= pDnprefs->FFT_SIZE/2 ; k++) {
 	double freq = (double)pPrefs->rate / 2.0 /(double)(pDnprefs->FFT_SIZE/2)*(double)k ;
-	fprintf(fp, "%10lg %lg %lg\n", freq, left_noise_avg[k], right_noise_avg[k]) ;
+	fprintf(fp, "%10lg %12.1lf %12.1lf\n", freq, left_noise_avg[k], right_noise_avg[k]) ;
     }
 
     fclose(fp) ;
@@ -654,7 +845,11 @@ void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs
 #endif /* HAVE_FFTW3 */
 
     for(k = 0 ; k < pDnprefs->FFT_SIZE ; k++) {
-	window_coef[k] = fft_window(k,pDnprefs->FFT_SIZE, pDnprefs->window_type) ;
+	if(0 || pDnprefs->window_type == DENOISE_WINDOW_HANNING_OVERLAP_ADD) {
+	    window_coef[k] = fft_window(k,pDnprefs->FFT_SIZE, pDnprefs->window_type) ;
+	} else {
+	    window_coef[k] = 1.0 ;
+	}
 	left_noise_max[k] = 0.0 ;
 	right_noise_max[k] = 0.0 ;
 	left_noise_avg[k] = 0.0 ;
@@ -724,6 +919,9 @@ void get_noise_sample(struct sound_prefs *pPrefs, struct denoise_prefs *pDnprefs
 	right_noise_avg[k] /= (double)pDnprefs->n_noise_samples ;
     }
 
+    compute_bark_z(pDnprefs->FFT_SIZE, pPrefs->rate) ;
+    compute_johnston_gain(pDnprefs->FFT_SIZE, 0.0) ;
+
     if(pDnprefs->freq_filter) {
 	if(pDnprefs->estimate_power_floor) {
 	    double half_freq_w = (pDnprefs->max_sample_freq - pDnprefs->min_sample_freq)/2.0 ;
