@@ -462,7 +462,6 @@ void draw_sonogram(struct view *v, struct sound_prefs *pPrefs, GtkWidget *da, do
 #define MAXSW 2000
 #define MAXSH 400
     unsigned char level[2][MAXSW][MAXSH] ;
-    int in_click ;
 
     push_status_text("Building sonogram") ;
     update_status_bar(0.0,STATUS_UPDATE_INTERVAL,TRUE) ;
@@ -671,12 +670,14 @@ ly2_tbl[0] = 99999999;
        v->canvas_width, v->canvas_height); 
     gdk_image_destroy(image);
 
+#ifdef DRAW_FFT_CLICKS
     for(channel = 0 ; channel < 2 ; channel++) {
 	double mean_level[MAXSH] ;
 	double offset[MAXSW] ;
 	double hgt_sum ;
-	static double mean, stderr, var, cv, stddev ;
+	double mean, std_err, var, cv, stddev ;
 	long click_start, click_end ;
+	int in_click ;
 
 	for(k = 0 ; k < FFT_SIZE/2 ; k++) {
 	    mean_level[k] = 0.0 ;
@@ -696,7 +697,7 @@ ly2_tbl[0] = 99999999;
 
 	}
 
-	stats(offset, v->canvas_width, &mean, &stderr, &var, &cv, &stddev) ;
+	stats(offset, v->canvas_width, &mean, &std_err, &var, &cv, &stddev) ;
 
 	in_click = 0 ;
 
@@ -739,6 +740,7 @@ ly2_tbl[0] = 99999999;
 	    draw_a_line(da->style->white_gc, x, ly1, x+1, ly2, green_color, 0) ;
 	}
     }
+#endif /* DRAW_FFT_CLICKS */
 
 #ifdef HAVE_FFTW3
     FFTW(destroy_plan)(pLeft);
