@@ -552,8 +552,12 @@ void declick_with_sensitivity(double sensitivity)
     long first, last;
     char *result_msg;
 
+    repair_clicks =
+	gtk_toggle_button_get_active((GtkToggleButton *)detect_only_widget) == TRUE ? FALSE : TRUE;
+
     if (repair_clicks == TRUE)
 	start_save_undo("Undo declick", &audio_view);
+
     get_region_of_interest(&first, &last, &audio_view);
 
     push_status_text(repair_clicks ==
@@ -562,11 +566,6 @@ void declick_with_sensitivity(double sensitivity)
     click_data.max_clicks = MAX_CLICKS;
     click_data.n_clicks = 0;
 
-    repair_clicks =
-	gtk_toggle_button_get_active((GtkToggleButton *)
-				     detect_only_widget) ==
-	TRUE ? FALSE : TRUE;
-
     result_msg =
 	do_declick(&prefs, first, last, audio_view.channel_selection_mask,
 		   sensitivity, repair_clicks, &click_data,
@@ -1239,8 +1238,18 @@ void zoom_select(GtkWidget * widget, gpointer data)
     if ((file_processing == FALSE) && (file_is_open == TRUE)
 	&& (audio_playback == FALSE) && (cursor_playback == FALSE)) {
 	file_processing = TRUE;
-	audio_view.first_sample = audio_view.selected_first_sample;
-	audio_view.last_sample = audio_view.selected_last_sample;
+
+	if(audio_view.selected_first_sample == audio_view.selected_last_sample) {
+	    audio_view.first_sample = audio_view.selected_first_sample - prefs.rate+1 ;
+	    audio_view.last_sample = audio_view.selected_last_sample + prefs.rate ;
+	} else {
+	    audio_view.first_sample = audio_view.selected_first_sample;
+	    audio_view.last_sample = audio_view.selected_last_sample;
+	}
+
+	if(audio_view.selected_first_sample < 0) audio_view.selected_first_sample = 0 ;
+	if(audio_view.selected_last_sample > prefs.n_samples-1) audio_view.selected_last_sample = prefs.n_samples-1 ;
+
 	audio_view.selection_region = FALSE;
 	set_scroll_bar(prefs.n_samples - 1, audio_view.first_sample,
 		       audio_view.last_sample);
@@ -1265,16 +1274,29 @@ void select_all(GtkWidget * widget, gpointer data)
 
 void select_markers(GtkWidget * widget, gpointer data)
 {
-    /*
-       if(audio_view.marker_1 >= 0 && audio_view.marker_2 >= 0) {
-       audio_view.selected_first_sample = MIN(audio_view.marker_1, audio_view.marker_2) ;
-       audio_view.selected_last_sample = MAX(audio_view.marker_1, audio_view.marker_2) ;
-       audio_view.selection_region = TRUE ;
-       main_redraw(FALSE, TRUE) ;
-       } else {
-       warning("Both markers must be set to select between markers") ;
-       }
-     */
+    int i ;
+    long new_first = -1 ;
+    long new_last = -1 ;
+
+    for (i = 0; i < n_markers; i++) {
+	if(markers[i] < audio_view.selected_first_sample) {
+	    if(new_first == -1)
+		new_first = markers[i] ;
+	    else
+		new_first = MAX(new_first, markers[i]) ;
+	}
+	if(markers[i] > audio_view.selected_last_sample) {
+	    if(new_last == -1)
+		new_last = markers[i] ;
+	    else
+		new_last = MIN(new_last, markers[i]) ;
+	}
+    }
+
+    if(new_first != -1) audio_view.selected_first_sample = new_first ;
+    if(new_last != -1) audio_view.selected_last_sample = new_last ;
+
+    main_redraw(FALSE, TRUE);
 }
 
 void toggle_marker_at(long sample)
@@ -1594,6 +1616,89 @@ void open_wave_filename(void)
 
     gnome_flush();
 
+    {
+	int l;
+	struct sound_prefs tmp_prefs;
+
+           /* Close writes to wave_filename so put back the name of the
+              current open file */
+        strcpy(tmp,wave_filename);
+        strcpy(wave_filename, last_filename);
+	cleanup_and_close();
+        strcpy(wave_filename,tmp);
+
+           /* all this to store the last directory where a file was opened */
+        strcpy(tmp, wave_filename);
+        strcpy(pathname, dirname(tmp));
+        strcat(pathname, "/");
+
+        strcpy(tmp, wave_filename);
+        strcat(pathname, basename(tmp));
+        
+	l = strlen(wave_filename);
+
+	if (is_valid_audio_file(wave_filename)) {
+	    tmp_prefs = open_wavefile((char *) wave_filename, &audio_view);
+	    if (tmp_prefs.successful_open) {
+		prefs = tmp_prefs;
+		spectral_view_flag = FALSE;
+		if (prefs.wavefile_fd != -1) {
+#ifdef TRUNCATE_OLD
+		    audio_view.truncate_head = 0;
+		    audio_view.truncate_tail = (prefs.n_samples - 1);
+#endif /* TRUNCATE_OLD */
+		    audio_view.n_samples = prefs.n_samples;
+		    if (audio_view.first_sample == -1) {
+			audio_view.first_sample = 0;
+			audio_view.last_sample = (prefs.n_samples - 1);
+		    } else {
+			audio_view.first_sample =
+			    MIN(prefs.n_samples - 1,
+				audio_view.first_sample);
+			audio_view.first_sample =
+			    MAX(0, audio_view.first_sample);
+			audio_view.last_sample =
+			    MIN(prefs.n_samples - 1,
+				audio_view.last_sample);
+			audio_view.last_sample =
+			    MAX(0, audio_view.last_sample);
+		    }
+		    audio_view.selection_region = FALSE;
+		    file_is_open = TRUE;
+		    fill_sample_buffer(&prefs);
+
+		    /* display entire file data if this file changed since last edit session */
+		    if (strcmp(wave_filename, last_filename)) {
+			audio_view.first_sample = 0;
+			audio_view.last_sample = prefs.n_samples - 1;
+			strcpy(last_filename, wave_filename);
+			num_song_markers = 0;
+		    }
+
+		    set_scroll_bar(prefs.n_samples - 1,
+				   audio_view.first_sample,
+				   audio_view.last_sample);
+		    main_redraw(FALSE, TRUE);
+		} else {
+		    file_is_open = FALSE;
+		    warning("failed to open audio file");
+		}
+	    } else {
+		strcpy(wave_filename, last_filename);
+	    }
+	} else {
+	    file_is_open = FALSE;
+	    warning("No file selected, or file format not recognized");
+	}
+    }
+}
+
+void old_open_wave_filename(void)
+{
+    char tmp[512];
+
+    gnome_flush();
+
     /* all this to store the last directory where a file was opened */
     strcpy(tmp, wave_filename);
     strcpy(pathname, dirname(tmp));
@@ -1979,6 +2084,9 @@ GnomeUIInfo marker_menu[] = {
     GNOMEUIINFO_ITEM("Clear Markers",
 		     "Clear all markers in the current selection or view",
 		     clear_markers_in_view, NULL),
+    GNOMEUIINFO_ITEM_ACCEL("Expand selection to nearest markers",
+		     "Select region between two markers",
+		     select_markers, NULL, GDK_m),
     GNOMEUIINFO_ITEM("Mark Songs",
 		     "Find songs in current selection or view",
 		     mark_songs, NULL),
