@@ -44,7 +44,7 @@
 
 #define GWC_VERSION_MAJOR 0
 #define GWC_VERSION_MINOR 21
-#define VERSION "0.21-03"
+#define VERSION "0.21-05"
 #define GWC_POINT_HANDLE 0x01
 #define SBW  128	/* Sample Block Width, the number of audio sammples summarized in one block  */
 #define STATUS_UPDATE_INTERVAL 0.5	/* update status bar every 1/2  second on long edit operations */
@@ -74,7 +74,9 @@
 #define DENOISE_POWER_SPECTRAL_SUBTRACT 1
 #define DENOISE_EM 2
 #define DENOISE_LORBER 3
-#define DENOISE_MAGNITUDE_SPECTRAL_SUBTRACT 4
+#define DENOISE_WOLFE_GODSILL 4
+
+//#define DENOISE_MAGNITUDE_SPECTRAL_SUBTRACT -1
 
 /* defs for markers */
 #define MAX_MARKERS 200
