@@ -93,26 +93,28 @@ void cdrdao_toc_info(char *filename)
        char *title;
        char *fieldid;
        char *init;
+       int always_write;
        GtkWidget *widget;
     } album_info[] = 
     {
-       {"Language", "LANGUAGE", "EN", NULL},  /* This is special and must be first */
-       {"Title", "TITLE",  "", NULL},
-       {"Performer", "PERFORMER", "", NULL},
-       {"Songwriter", "SONGWRITER", "", NULL},
-       {"Arranger", "ARRANGER", "", NULL},
-       {"Composer", "COMPOSER", "", NULL},
-       {"Disc_ID", "DISC_ID", "", NULL},
-       {"Message", "MESSAGE", "", NULL}
+       {"Language", "LANGUAGE", "EN", 1, NULL},  /* This is special and must be first */
+       {"Title", "TITLE",  "", 1, NULL},
+       {"Performer", "PERFORMER", "", 0, NULL},
+       {"Songwriter", "SONGWRITER", "", 0, NULL},
+       {"Arranger", "ARRANGER", "", 0, NULL},
+       {"Composer", "COMPOSER", "", 0, NULL},
+       {"Disc_ID", "DISC_ID", "", 0, NULL},
+       {"Message", "MESSAGE", "", 1, NULL}
     };
     struct {
        char *title;
        char *fieldid;
        char *init;
+       int always_write;
     } song_info[] = 
     {
-       {"Title", "TITLE",  ""},
-       {"Message", "MESSAGE",  ""}
+       {"Title", "TITLE",  "", 1},
+       {"Message", "MESSAGE",  "", 1}
     };
     GtkWidget *song_widget[MAX_MARKERS+1][ARRAYSIZE(song_info)];
     int i,j;
@@ -212,7 +214,7 @@ void cdrdao_toc_info(char *filename)
 	      for (i = 0; i < ARRAYSIZE(album_info); i++) {
 		 char *str;
 		 str = (char *)gtk_entry_get_text(GTK_ENTRY(album_info[i].widget));
-		 if (!only_blank(str)) {
+		 if (album_info[i].always_write || !only_blank(str)) {
 		       /* First entry, language added to file above */
 		    if (i > 0) {
 		       fprintf(toc, "      %s \"%s\"\n", album_info[i].fieldid, str);
@@ -229,16 +231,19 @@ void cdrdao_toc_info(char *filename)
 	   }
 
 	   for (j = 0; j < num_song_markers+1; j++) {
-	      int end = song_markers[j];
-	      if (j == num_song_markers)
+	      int end;
+	      if (j == num_song_markers) {
 		 end = prefs.n_samples-1;
+              } else {
+                 end = ((song_markers[j] + 2324 / 2) / 2324) * 2324;
+              }
 	      fprintf(toc, "TRACK AUDIO\n");
 	      if (found_text) {
 		 fprintf(toc, "   CD_TEXT {\n      LANGUAGE 0 {\n");
 		 for (i = 0; i < ARRAYSIZE(song_info); i++) {
 		    char *str;
 		    str = (char *)gtk_entry_get_text(GTK_ENTRY(song_widget[j][i]));
-		    if (!only_blank(str)) {
+		    if (song_info[i].always_write || !only_blank(str)) {
 		       fprintf(toc, "         %s \"%s\"\n", song_info[i].fieldid, str);
 			  /* Duplicated above */
 		       snprintf(buf, sizeof(buf), "%3d%s", j+1, song_info[i].fieldid);
