@@ -1071,6 +1071,8 @@ void noise_method_window_select(GtkWidget * clist, gint row, gint column,
 	noise_suppression_method = DENOISE_EM;
     if (row == 3)
 	noise_suppression_method = DENOISE_LORBER;
+    if (row == 4)
+	noise_suppression_method = DENOISE_WOLFE_GODSILL;
 }
 
 void denoise_set_preferences(GtkWidget * widget, gpointer data)
@@ -1094,17 +1096,19 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
     gchar *fft_window_titles[] = { "Windowing Function" };
     gchar *fft_window_parms[3][1] = { {"Blackman"},
     {"Hybrid Blackman-Full Pass"},
-    {"Hanning-overlap-add"}
+    {"Hanning-overlap-add (Best)"}
     };
 
     GtkWidget *noise_method_window_list;
 
     gchar *noise_method_window_titles[] = { "Noise Suppresion Method" };
-    gchar *noise_method_window_parms[4][1] = { {"Weiner"},
-    {"Power Spectral Subtraction"},
-    {"Ephraim-Malah (1984)"},
-    {"Lorber & Hoeldrich"}
-    };
+    gchar *noise_method_window_parms[5][1] = {
+						{"Weiner"},
+						{"Power Spectral Subtraction"},
+						{"Ephraim-Malah 1984"},
+						{"Lorber & Hoeldrich (Best)"},
+						{"Wolfe & Godsill (Experimental)"}
+					    };
 
     load_denoise_preferences();
 
@@ -1134,6 +1138,8 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 		     noise_method_window_parms[2]);
     gtk_clist_append(GTK_CLIST(noise_method_window_list),
 		     noise_method_window_parms[3]);
+    gtk_clist_append(GTK_CLIST(noise_method_window_list),
+		     noise_method_window_parms[4]);
     gtk_clist_select_row(GTK_CLIST(noise_method_window_list),
 			 denoise_prefs.noise_suppression_method, 0);
     gtk_signal_connect(GTK_OBJECT(noise_method_window_list), "select_row",
@@ -1156,7 +1162,7 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 
     fft_size_entry =
 	add_number_entry_with_label_int(denoise_prefs.FFT_SIZE,
-					"FFT_SIZE:", dialog_table, 0);
+					"FFT_SIZE (4096 for 44.1khz sample rate)", dialog_table, 0);
     amount_entry =
 	add_number_entry_with_label_double(denoise_prefs.amount,
 					   "Reduction (0.0-1.0)",
@@ -1172,7 +1178,7 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 
     gamma_entry =
 	add_number_entry_with_label_double(denoise_prefs.dn_gamma,
-					   "gamma -- for Lorber & Hoelrich and Ephraim-Malah , (0-1, try 0.98)",
+					   "gamma -- for Lorber & Hoelrich or Ephraim-Malah , (0.9-1, try 0.98)",
 					   dialog_table, 4);
 
     freq_filter_entry =
